package com.ibm.itso.sal404.interestlist;

import java.util.List;

import org.apache.log4j.Logger;

import com.ibm.itso.sal404.common.base.ManagerIf;
import com.ibm.itso.sal404.common.exception.ApplicationException;
import com.ibm.itso.sal404.common.util.BooleanValueHelper;
import com.ibm.itso.sal404.interestlist.dao.CheckoutInterestListDAO;
import com.ibm.itso.sal404.interestlist.dao.CheckoutInterestListItemDAO;
import com.ibm.itso.sal404.interestlist.dao.InterestListPropertyDetailDAO;
import com.ibm.itso.sal404.interestlist.dao.InterestListPropertyDetailDAORow;
import com.ibm.itso.sal404.interestlist.dto.InterestListDTO;
import com.ibm.itso.sal404.interestlist.itemadapter.InterestListLineItem;
import com.ibm.itso.sal404.user.dto.UserDetailsDTO;

/**
 * 
 * sal404 Realty sample application
 * 
 * @component	Interest list
 * @layer		Business logic layer
 * 
 * This class acts as a business facade for the interest list component.
 * 
 * @author 		Mark Buensalido
 * @created		14 May 2003		
 * @version		1.0.0 / @14-May-03	
 *
 * 
 */
public class InterestListManager implements ManagerIf
{
	/**
	 * Configures the log4j Logger.
	 * @since 1.0.0
	 */
	private static Logger logger = Logger.getLogger(InterestListManager.class);

	/**
	 * Field for the InterestListDTO.
	 */
	private InterestListDTO _interestListDTO = null;

	/**
	 * Field for the UserDTO.
	 */
	private UserDetailsDTO _userdetailsDTO = null;

	/**
	 * Constructs an InterestListManager.
	 * @param InterestListDTO interestListDTO
	 * @since 1.0.0
	 */
	public InterestListManager(InterestListDTO interestListDTO)
	{
		super();
		setInterestListDTO(interestListDTO);
		logger.info("ENTRY:InterestListManager(interestListDTO)");

	}

	/**
	 * Returns a new instance of the InterestListDTO based on userDTO.
	 * @param UserDTO userDTO
	 * @since 1.0.0
	 */
	public static InterestListDTO getNewInterestList(UserDetailsDTO userDetailsDTO)
	{
		return new InterestListDTO();
	}
	/**
	 * Returns the interestListDTO.
	 * @return InterestListDTO
	 * @since 1.0.0
	 */
	private InterestListDTO getInterestListDTO()
	{

		return _interestListDTO;
	}

	/**
	 * Sets the interestListDTO.
	 * @param interestListDTO The interestListDTO to set
	 * @since 1.0.0
	 */
	private void setInterestListDTO(InterestListDTO interestListDTO)
	{
		this._interestListDTO = interestListDTO;
	}

	/**
	 * Implements the "checkout" function of the Interest List Component.
	 * @returns boolean
	 * @since 1.0.0
	 */
	public void checkout(Integer userId) throws ApplicationException
	{

		CheckoutInterestListDAO listDAO = null;
		CheckoutInterestListItemDAO itemDAO = null;
		InterestListDTO DTO = null;
		List items = null;
		InterestListLineItem itemDTO = null;
		int newListId;
		String cEmail = "", cBrochure = "";

		listDAO = new CheckoutInterestListDAO();
		itemDAO = new CheckoutInterestListItemDAO();

		DTO = getInterestListDTO();
		items = DTO.getInterestListLineItems();


		logger.info("ENTRY:checkout");
		// if the size of the interest list is zero an exception is thrown
		if (items.size() == 0)
		{
			throw new ApplicationException("interestList.error.checkout.emptyList");
		}

		// this is the attempt to create the main interest list in the database
		try
		{
			logger.debug("Before add new interest list for:" + userId.toString());
			listDAO.execute(userId);
		}
		catch (Exception e)
		{
			logger.error(e.getClass().getName() + ":" + e.getMessage());
			throw new ApplicationException("interestList.error.checkout.databaseAccess");
		}

		// this is the database assigned Id for the new interest list
		newListId = listDAO.getNewIntListId();

		// iterate the interest list items and create them in the database
		try
		{
			for (int lcv = 0; lcv < items.size(); lcv++)
			{
				itemDTO = (InterestListLineItem) items.get(lcv);
				cEmail =
					BooleanValueHelper.convertBooleanToYNString(
						itemDTO.isEmailRequested());
				cBrochure =
					BooleanValueHelper.convertBooleanToYNString(
						itemDTO.isBrochureRequested());
				
				logger.debug("ItemDTO:" + lcv);
				
				itemDAO.execute(
					new Integer(newListId),
					new Integer(itemDTO.getPropertyId()),
					cEmail,
					cBrochure,
					"");
			}
		}
		catch (Exception e)
		{
			logger.error(e.getClass().getName() + ":" + e.getMessage());
			throw new ApplicationException("interestList.error.checkout.databaseAccess");
		}

		logger.info("EXIT:checkout");

	}

	/**
	 * Clears the interest list.
	 * @return InterestListDTO
	 * @since 1.0.0
	 */
	public InterestListDTO clear()
	{
		return _interestListDTO = new InterestListDTO();
	}

	/**
	 * Adds an item to the Interest List.
	 * @param propertyID ID of the property to be added.
	 * @exception ApplicationException
	 * @since 1.0.0
	 */
	public void addItem(String propertyId) throws ApplicationException
	{

		InterestListDTO DTO = null;
		InterestListLineItem DTOLine = null;
		InterestListPropertyDetailDAO DAO = null;
		InterestListPropertyDetailDAORow detailRow = null;
		Integer nPropertyID;
		String address = "", type = "", status = "", price = "", agentName = "";

		// write entry to the log
		logger.info("ENTRY:addItem(propertyId)");

		// instantiate the DAO to retrieve the property information
		DAO = new InterestListPropertyDetailDAO(1);
		logger.debug("InterestListPropertyDetailDAO instantiated");

		// check for a valid propertyId string
		if (propertyId == null)
		{
			logger.warn("PropertyId is null.");
			throw new ApplicationException("interestList.error.addItem.invalidId");
		}

		logger.debug("Before Number Cast PropertyId is:" + propertyId);

		// check for an appropriate format for the number
		try
		{
			nPropertyID = new Integer(propertyId);
		}
		catch (NumberFormatException e)
		{
			logger.warn("NumberFormatException:" + e.getMessage());
			throw new ApplicationException("interestList.error.addItem.invalidId");
		}

		// check to make sure that the property is not already on the interest list
		logger.debug("Checking for duplicate");
		DTO = this.getInterestListDTO();
		for (int lcv = 0; lcv < DTO.getInterestListLineItems().size(); lcv++)
		{
			DTOLine = DTO.getInterestListLineItem(lcv);
			logger.debug("Checking:" + DTOLine.getPropertyId());
			if (DTOLine.getPropertyId() == nPropertyID.intValue())
			{
				logger.debug("Duplicate found");
				throw new ApplicationException("interestList.error.addItem.duplicateItem");
			}
		}
		logger.debug("End of duplicate check");

		try
		{

			// execute the DAO
			DAO.execute(new Integer[] { nPropertyID });
			logger.debug("DAO Executed");

			// retrieve the detail row			
			detailRow = DAO.getRows()[0];

		}
		catch (Exception e)
		{
			logger.error(e.getClass().getName() + ":" + e.getMessage());
			throw new ApplicationException("interestList.error.addItem.databaseAccess");
		}

		// build the content
		if (detailRow != null)
		{

			// address
			try
			{
				address += detailRow.getPROP_PROP_ADDR_BLDG_NUM().toString();
			}
			catch (Exception e)
			{
				address += "";
			}

			// space out the building number and street name
			address += " ";

			try
			{
				address += detailRow.getPROP_PROP_ADDR_STRT_NAM().toString();
			}
			catch (Exception e)
			{
				address += "";
			}
			logger.debug("Address Retrieved: " + address);

			// build the type
			try
			{
				type += detailRow.getPROP_TYPE_PROP_TYPE_DESC().toString();
			}
			catch (Exception e)
			{
				type += "";
			}
			logger.debug("Type Retrieved: " + type);

			// build the price
			try
			{
				price += detailRow.getPROP_PROP_PRICE().toString();
			}
			catch (Exception e)
			{
				price += "";
			}
			logger.debug("Price Retrieved: " + price);

			// build the agent name
			try
			{
				agentName += detailRow.getUSR_USR_FIRST_NAME().toString();
			}
			catch (Exception e)
			{
				agentName += "";
			}

			// space out the first name and last name
			agentName += " ";

			try
			{
				agentName += detailRow.getUSR_USR_LAST_NAME().toString();
			}
			catch (Exception e)
			{
				agentName += "";
			}
			logger.debug("Agent Name Retrieved: " + agentName);
		}

		// fill the DTO line item with information
		DTOLine = new InterestListLineItem();
		DTOLine.setAddress(address);
		DTOLine.setType(type);
		DTOLine.setPrice(price);
		DTOLine.setAgentName(agentName);
		DTOLine.setPropertyId(nPropertyID.intValue());

		DTO.getInterestListLineItems().add(DTOLine);
		logger.debug("DTOLine added");

	}

	/**
	 * Updates the item
	 * @param int index
	 * @since 1.0.0
	 */
	public void updateItem(InterestListLineItem item)
		throws ApplicationException
	{

		InterestListDTO DTO = null;
		InterestListLineItem dtoItem = null;

		DTO = this.getInterestListDTO();

		try
		{

			// retrieves the DTO item represented by the propertyId
			dtoItem = getDTOItem(item.getPropertyId());

			// sets the item's parameters
			if (item.isRemove())
			{
				DTO.getInterestListLineItems().remove(dtoItem);
			}
			else
			{
				dtoItem.setBrochureRequested(item.isBrochureRequested());
				dtoItem.setEmailRequested(item.isEmailRequested());
			}
		}
		catch (Exception e)
		{
			throw new ApplicationException("interestList.error.updateItem.nullItem");
		}

	}

	/**
	 * Retrieves a DTO named by a propertyId.
	 * @param int propertyId
	 * @return InterestListLineItem
	 * @since 1.0.0
	 */
	private InterestListLineItem getDTOItem(int propertyId)
	{

		InterestListDTO DTO = null;
		List items = null;
		InterestListLineItem item = null;

		DTO = this.getInterestListDTO();
		items = DTO.getInterestListLineItems();

		// scans for the InterestListLineItem
		for (int lcv = 0; lcv < items.size(); lcv++)
		{
			item = (InterestListLineItem) items.get(lcv);
			if (item.getPropertyId() == propertyId)
			{
				break;
			}
			else
			{
				item = null;
			}
		}

		return item;
	}

}
