/* $Id: MessageGateway.java,v 1.3 2004/12/14 15:29:02 grove Exp $
 * 
 * This code is part of the sample application that has been created
 * for the redbook WebSphere Application Server - Express V6
 * Developers Guide and A Development Example (SG24-6500).
 *
 * Revision History:
 *
 *   $Log: MessageGateway.java,v $
 *   Revision 1.3  2004/12/14 15:29:02  grove
 *   refactoring
 *
 *   Revision 1.1  2004/11/25 14:47:15  grovedc
 *   week 4
 *
 *   Revision 1.2  2004/11/22 18:09:11  ansgar
 *   added comments
 *
 *   Revision 1.1  2004/11/22 15:14:53  ansgar
 *   Initial Revision
 *
 */
package com.ibm.itso.sal404.jms;

import java.io.Serializable;
import java.util.HashMap;

import javax.jms.Connection;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import com.ibm.itso.sal404.common.util.AppProperties;

/**
 * This gateway can be used to send JMS messages.
 * 
 * It is implemented as a singleton that caches the JMS settings. If you change
 * the administered objects like the connection factory or the queue in the
 * WebSphere configuration (via the admin console) then you need to restart the
 * server in order to refresh the cached values in the singleton.
 * 
 * <BR>
 * <BR>
 * Creation date: Nov 21, 2004
 * 
 * @author Ansgar Hugo
 */
public class MessageGateway {

    /** log4j logger */
    private static Logger logger = Logger.getLogger(MessageGateway.class);

    /** the instance of this singleton */
    private static MessageGateway instance = null;

    /** holds any settings for the gateway */
    private HashMap _settings;

    private static final String DEFAULT_CONNECTION_FACTORY_NAME = "jms/RealtyConnectionFactory";

    private static final String DEFAULT_QUEUE_NAME = "jms/RealtyIncomingQueue";

    private String connectionFactoryName = null;

    /** queue name */
    private String queueName = null;

    /** cached JNDI context */
    private Context context;

    /** cached connectionFactory */
    private QueueConnectionFactory connectionFactory;

    /** cached queue */
    private Queue queue;

    /**
     * flag that indicates wether jndi lookup uses java:comp/env or global JNDI
     * lookup name space. if set to false there must be a local jndi reference
     * configured in the settings of the web project. Gets overwritten by the
     * loadProperties() method.
     */
    private boolean globalLookup = false;

    /**
     * Empty private constructor initializes the gateway.
     * 
     * @throws NamingException
     */
    private MessageGateway() throws NamingException {
        try {
            init();
        } catch (NamingException e) {
            logger.error("MessageGateway() cannot initialize MessageGateway,"
                    + " unable to find JMS resources via JNDI.", e);
            throw e;
        }
    }

    /**
     * Does the intitialization of the gateway settings.
     * 
     * @throws NamingException
     */
    private void init() throws NamingException {
        logger.debug("init() ENTER");

        loadProperties();

        if (globalLookup) {
            context = new InitialContext();
        } else {
            InitialContext initialContext = new InitialContext();
            context = (Context) context.lookup("java:comp/env");
        }
        logger.debug("init() got env context");
        lookupConnectionFactory();
        lookupQueue();

        logger.debug("init() EXIT");
    }

    /**
     *  
     */
    private void loadProperties() {
        // get the configuration properties
        AppProperties properties = AppProperties.getInstance();
        connectionFactoryName = properties
                .getProperty(AppProperties.JMS_JNDI_CONNECTION_FACTORY);
        queueName = properties.getProperty(AppProperties.JMS_JNDI_OUT_QUEUE);
        globalLookup = Boolean.valueOf(
                properties.getProperty(AppProperties.JMS_JNDI_GLOBAL))
                .booleanValue();
    }

    /**
     * Does the JNDI lookup of the connection factory.
     * 
     * @throws NamingException
     */
    private void lookupConnectionFactory() throws NamingException {
        connectionFactory = (QueueConnectionFactory) context
                .lookup(getConnectionFactoryName());
    }

    /**
     * Does the JNDI lookup of the queue.
     * 
     * @throws NamingException
     */
    private void lookupQueue() throws NamingException {
        queue = (Queue) context.lookup(getQueueName());
    }

    /**
     * Creates the instance of this Singleton.
     */
    private static synchronized void createInstance() {
        if (instance == null) {
            try {
                instance = new MessageGateway();
            } catch (NamingException e) {
                logger.error("createInstance() could not create instance", e);
            }
        }
    }

    /**
     * Returns an instance of the MessageGateway singleton.
     * 
     * @return the <code>MessageGateway</code> instance
     */
    public static MessageGateway getInstance() {
        if (instance == null) {
            createInstance();
        }
        return instance;
    }

    /**
     * Destroys the instance of this singleton. If needed you can release any
     * resources here.
     */
    public static synchronized void destroyInstance() {
        instance = null;
    }

    /**
     * @return Returns the connectionFactoryName.
     */
    private String getConnectionFactoryName() {
        return connectionFactoryName;
    }

    /**
     * @param connectionFactoryName
     *            The connectionFactoryName to set.
     */
    public synchronized void setConnectionFactoryName(String connectionFactoryName) {
        this.connectionFactoryName = connectionFactoryName;
        try {
            lookupConnectionFactory();
        } catch (NamingException e) {
            logger
                    .error(
                            "setConnectionFactoryName() cannot find connection factory, leaving old settings",
                            e);
        }
    }

    /**
     * @return Returns the queueName.
     */
    private String getQueueName() {
        return queueName;
    }

    /**
     * @param queueName
     *            The queueName to set.
     */
    public synchronized void setQueueName(String queueName) {
        this.queueName = queueName;
        try {
            lookupQueue();
        } catch (NamingException e) {
            logger.error("setQueueName() cannot find queue, "
                    + "leave old settings", e);
        }
    }

    /**
     * Sends an object to the JMS queue that is specified in the config settings
     * of the application.
     * 
     * @param object -
     *            the object to send
     */
    public synchronized void sendMessage(Serializable object) {
        logger.debug("sendMessage() ENTER");
        try {
            // get the connection from the factory
            Connection connection = connectionFactory.createConnection();
            // create a transacted session
            Session session = connection.createSession(true,
                    QueueSession.AUTO_ACKNOWLEDGE);
            // create the message producer for the queue
            MessageProducer messageProducer = session.createProducer(queue);
            // create the object message
            ObjectMessage message = session.createObjectMessage();
            
            message.setObject(object);

            // send message
            messageProducer.send(message);
            // commit transaction
            session.commit();
            // close producer
            messageProducer.close();
            // close connection
            connection.close();
        } catch (JMSException e) {
            logger.error("sendMessage() error sending JMS message", e);
        }
        logger.debug("sendMessage() EXIT");
    }

}