package com.ibm.itso.sal404.user;

import org.apache.log4j.Logger;

import com.ibm.itso.sal404.common.base.ManagerIf;
import com.ibm.itso.sal404.common.exception.ApplicationException;
import com.ibm.itso.sal404.common.util.AppProperties;
import com.ibm.itso.sal404.jms.MessageGateway;
import com.ibm.itso.sal404.user.dto.UserDetailsDTO;
import com.ibm.itso.sal404.user.dto.UserSearchCriteriaDTO;
import com.ibm.itso.sal404.user.dto.UserVectorDTO;

/**
 * 
 * sal404 Realty sample application
 * 
 * @component	User
 * @layer		Manager
 * 
 * This class is the fascade to the user manager implementation.  It simply
 * calls either the LDAP or the database implementation.  And yes, I know that
 * I should have used an interface and a factory.  The code has been kept simple
 * for clarity.
 * 
 * @author 		Doug Grove
 * @created		4/15/2003	
 * @version		x.y.z / @${date}	
 *
 * @see		 	classes associated with this class
 * 
 */
public class UserManager implements ManagerIf
{
	private boolean useLDAP = true;

	private String userDS = null;

	// LDAP
	private UserManagerLDAP userManagerLDAP = new UserManagerLDAP();

	// Database
	private UserManagerDB userManagerDB = new UserManagerDB();

	// Configure Log4J Logger
	private static Logger logger = Logger.getLogger(UserManager.class);

	/**
	 * Constructor for UserManager.
	 */
	public UserManager()
	{
		super();

		userDS = AppProperties.getInstance().getProperty("userDataSource");

		if (userDS.equalsIgnoreCase("ldap"))
			useLDAP = true;

		if (userDS.equalsIgnoreCase("database"))
			useLDAP = false;
	}

	/**
	 * 
	 */
	public UserDetailsDTO login(UserDetailsDTO loginDTO)
		throws ApplicationException
	{
		UserDetailsDTO UserDetailsDTO = null;

		logger.info("ENTRY: login");

		if (useLDAP)
			UserDetailsDTO = getUserManagerLDAP().login(loginDTO);
		else
			UserDetailsDTO = getUserManagerDB().login(loginDTO);

		logger.info("EXIT:  login");

		return UserDetailsDTO;
	}

	/**
	 * 
	 */
	public UserVectorDTO search(UserSearchCriteriaDTO userSearchCriteriaDTO)
		throws ApplicationException
	{
		UserVectorDTO userVectorDTO = null;

		logger.info("ENTRY: search");

		if (useLDAP)
		{
			userVectorDTO = getUserManagerLDAP().search(userSearchCriteriaDTO);
		}
		else
		{
			userVectorDTO = getUserManagerDB().search(userSearchCriteriaDTO);
		}

		logger.info("EXIT:  search");
		return userVectorDTO;
	}

	/**
	 * 
	 */
	public void modifyUserDetails(UserDetailsDTO userDetailsDTO)
		throws ApplicationException
	{
		logger.info("ENTRY: modifyUserDetails");

		if (useLDAP)
		{

			getUserManagerDB().modifyUserDetails(userDetailsDTO);
			getUserManagerLDAP().modifyUserDetails(userDetailsDTO);
		}
		else
			getUserManagerDB().modifyUserDetails(userDetailsDTO);

		// send the userDetailsDTO to a JMS queue
		sendUserDetails(userDetailsDTO);
		
		logger.info("EXIT:  modifyUserDetails");
	}

	/**
	 * Uses the MessageGateway to send the user details to a
	 * JMS queue.
     * @param userDetailsDTO
     */
    private void sendUserDetails(UserDetailsDTO userDetailsDTO) {
        MessageGateway messageGateway = MessageGateway.getInstance();
        messageGateway.sendMessage(userDetailsDTO);
        
    }

    /**
	 * 
	 */
	public UserDetailsDTO registerUser(UserDetailsDTO userDetailsDTO)
		throws ApplicationException
	{
		logger.info("ENTRY: registerUser");
		UserDetailsDTO _userDetailsDTO = null;

		if (useLDAP)
		{
			_userDetailsDTO = getUserManagerDB().registerUser(userDetailsDTO);
			_userDetailsDTO = getUserManagerDB().getIdForUser(_userDetailsDTO);
			getUserManagerLDAP().registerUser(_userDetailsDTO);

		}
		else
			_userDetailsDTO = getUserManagerDB().registerUser(userDetailsDTO);

		logger.info("EXIT:  registerUser");
		return _userDetailsDTO;
	}

	/**
	 * 
	 */
	public UserDetailsDTO getUserByName(String userName)
		throws ApplicationException
	{
		logger.info("ENTRY: getUserByName");
		UserVectorDTO userVectorDTO = null;

		if (useLDAP)
		{
			userVectorDTO = getUserManagerLDAP().buildUserDetailsDTO(userName);
		}
		else
		{
			userVectorDTO = getUserManagerDB().buildUserDetailsDTO(userName);
		}

		logger.info("EXIT:  getUserByName");

		return (UserDetailsDTO) userVectorDTO.getUserVector().elementAt(0);
	}

	/**
	 * 
	 * 
	 */
	public UserVectorDTO buildUserDetailsDTO(String userName)
		throws ApplicationException
	{
		logger.info("ENTRY: buildUserDetailsDTO");
		UserVectorDTO userVectorDTO = null;

		if (useLDAP)
		{
			userVectorDTO = getUserManagerLDAP().buildUserDetailsDTO(userName);
		}
		else
		{
			userVectorDTO = getUserManagerDB().buildUserDetailsDTO(userName);
		}

		logger.info("EXIT:  buildUserDetailsDTO");
		return userVectorDTO;
	}

	/**
	 * 
	 * 
	 */
	public void deleteUser(UserDetailsDTO userDetails)
	{
		logger.info("ENTRY: deleteUser");

		if (useLDAP)
		{

			getUserManagerDB().deleteUser(userDetails);
			getUserManagerLDAP().deleteUser(userDetails);
		}
		else
		{
			getUserManagerDB().deleteUser(userDetails);
		}

		logger.info("EXIT:  deleteUser");
	}

	/**
	 * Returns the userManagerLDAP.
	 * @return UserManagerLDAP
	 */
	public UserManagerLDAP getUserManagerLDAP()
	{
		return userManagerLDAP;
	}

	/**
	 * Sets the userManagerLDAP.
	 * @param userManagerLDAP The userManagerLDAP to set
	 */
	public void setUserManagerLDAP(UserManagerLDAP userManagerLDAP)
	{
		this.userManagerLDAP = userManagerLDAP;
	}

	/**
	 * Returns the userManagerDB.
	 * @return UserManagerDB
	 */
	public UserManagerDB getUserManagerDB()
	{
		return userManagerDB;
	}

	/**
	 * Sets the userManagerDB.
	 * @param userManagerDB The userManagerDB to set
	 */
	public void setUserManagerDB(UserManagerDB userManagerDB)
	{
		this.userManagerDB = userManagerDB;
	}

	public UserVectorDTO getAllAgents() throws ApplicationException
	{
		return getUsersInRole("Agents");

	}

	public UserVectorDTO getAllAdministrators() throws ApplicationException
	{
		return getUsersInRole("Administrators");

	}
	public UserVectorDTO getAgentsAndAdministrators()
		throws ApplicationException
	{
		UserVectorDTO all = new UserVectorDTO();
		all = getAllAgents();
		all.getUserVector().addAll(getAllAdministrators().getUserVector());
		return all;

	}

	private UserVectorDTO getUsersInRole(String roleName)
		throws ApplicationException
	{
		logger.info("ENTRY: getUsersInRole");
		UserVectorDTO usersInRole = null;

		if (useLDAP)
		{

			usersInRole = getUserManagerLDAP().getUsersInRole(roleName);

		}
		else
		{
			usersInRole = getUserManagerDB().getUsersInRole(roleName);
		}

		logger.info("EXIT: getUsersInRole");
		return usersInRole;

	}
}
