package com.ibm.itso.sal404.common.util;

import java.sql.Connection;
import java.sql.SQLException;
import javax.sql.DataSource;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.ibm.itso.sal404.common.exception.ApplicationException;


/**
 * 
 * sal404 Realty sample application
 * 
 * @component	CommonComponent
 * @layer		N/A
 * 
 * This is a helper class used to obtain database connections from a datasource.
 * 
 * 1. 	It creates an instance of the InitialContext (if one doesn't already exist).
 * 2. 	It performs a lookup on the JNDI provider for a datasource with the name specified 
 * 		in the app.properties file (if an instance of the datasource hasn't been already
 * 		obtained from a previous lookup).
 * 3.	It obtains a database connection from the datasource.
 *
 * 
 * @author 		Stelios Psarras, Cyantific.net Ltd
 * @created		13 May 2003		
 * @version		1.0.0 / @13-May-03	
 *
 * @see		 	J2SE API documentation,
 * 				J2EE API documentation	
 * 
 */
public class DatabaseConnectionFactory
{
	private InitialContext	_initialContext = null;			// the InitialContext to be used
	private DataSource		_dataSource = null;				// a holder for the DataSource that will be obtained
	private String			_dataSourceJNDIName = null;		// the JNDI name of the DataSource
	
	
	/** 
	 * 
	 * Constructor for the DatabaseConnectionFactory
	 * 
	 * It reads the dataSourceJNDIName property from the app.properties file and 
	 * sets the value of _dataSourceJNDIName variable
	 * 
	 * @author Stelios Psarras, Cyantific.net Ltd
	 * 
	 */
	public DatabaseConnectionFactory()
	{
		super();
		_dataSourceJNDIName = AppProperties.getInstance().getProperty("dataSourceJNDIName");
	}
	
	
	/**
	 * 
	 * getInitialContext()
	 * 
	 * It creates an instance of the InitialContext (if one doesn't already exist).
	 * 
	 * @author Stelios Psarras, Cyantific.net Ltd
	 * @return InitialContext
	 * @exception NamingException
	 * 
	 */
	private InitialContext getInitialContext() throws NamingException
	{	
		if (_initialContext == null)
		{
			_initialContext = new InitialContext();
		}
		return _initialContext;
	}	
	
	
	/**
	 * 
	 * getDataSource()
	 * 
	 * It performs a lookup on the JNDI provider for a datasource with the name specified 
	 * in the app.properties file (if an instance of the datasource hasn't been already
	 * obtained from a previous lookup).
	 * 
	 * @author Stelios Psarras, Cyantific.net Ltd
	 * @return DataSource
	 * @exception NamingException, SQLException
	 * 
	 */
	private DataSource getDataSource() throws NamingException, SQLException
	{
		if (_dataSource == null)
		{
			_dataSource = (DataSource) getInitialContext().lookup(_dataSourceJNDIName);
		}
		return _dataSource;
	}


	/**
	 * 
	 * getConnection()
	 * 
	 * It obtains a database connection from the datasource.
	 * 
	 * @author Stelios Psarras, Cyantific.net Ltd
	 * @return DataSource
	 * @exception ApplicationException
	 * 
	 */
	public Connection getConnection() throws ApplicationException
	{
		Connection connection  = null;

		try 
		{
			if (getDataSource() != null)
			{
				connection = getDataSource().getConnection();
			}
		}
		catch (NamingException ne)
		{
			throw new ApplicationException(ne.getMessage());
		}
		catch (SQLException sqle) 
		{
			throw new ApplicationException(sqle.getMessage());
		}
		return connection;
	}
}

